/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <recorder.h>
#include <Elementary.h>
#include <efl_extension.h>
#include <system_settings.h>
#include <app.h>
#include <dlog.h>

#include "$(appName).h"
#include "view.h"
#include "data.h"

/* This means 1HOUR */
#define RECORDING_TIME_LIMIT 60*60
#define EDJ_FILE "edje/main.edj"

typedef struct {
	char *name;
	char *sub_name;
	char *icon;
} more_item_info;

typedef enum {
	PREPARE = 0,
	START,
	PAUSE,
	CANCEL,
	SAVE,
} recorder_control_type_e;

static more_item_info main_more_item[MAIN_MORE_ITEM_MAX] = {
	{"List", "", "image/more_option_icon_stt_list.png"},
	{"Text", "Input", "image/more_option_icon_stt_on.png"},
};

static more_item_info detail_more_item[DETAIL_MORE_ITEM_MAX] = {
	{"Remove", "", "image/more_option_icon_delete.png"},
};

static struct main_info {
	Elm_Object_Item *main_view_item;
	Elm_Object_Item *record_view_item;
	Elm_Object_Item *genlist_item;
	Evas_Object *record_view;
	recorder_h recorder;
	Ecore_Timer *pause_effect_timer;
	int effect_count;
} s_info = {
	.main_view_item = NULL,
	.record_view_item = NULL,
	.genlist_item = NULL,
	.record_view = NULL,
	.recorder = NULL,
	.pause_effect_timer = NULL,
	.effect_count = 0,
};

static void _main_clicked_cb(void *user_data, Evas_Object *obj, void *event_info);
static void _rec_stop_clicked_cb(void *user_data, Evas_Object *obj, void *event_info);
static void _rec_cancel_clicked_cb(void *user_data, Evas_Object *obj, void *event_info);
static void _rec_pause_clicked_cb(void *user_data, Evas_Object *obj, void *event_info);
static void _main_more_item_clicked_cb(void *user_data, Evas_Object *obj, void *event_info);
static void _detail_more_item_clicked_cb(void *user_data, Evas_Object *obj, void *event_info);
static void _detail_play_clicked_cb(void *user_data, Evas_Object *obj, void *event_info);

static Eina_Bool _naviframe_record_view_pop_cb(void *user_data, Elm_Object_Item *it);
static Eina_Bool _naviframe_main_view_pop_cb(void *user_data, Elm_Object_Item *it);

static void _voice_memo_initialize_recorder(void);
static void _voice_memo_deinitialize_recorder(void);
static Eina_Bool _voice_memo_control_recorder(recorder_control_type_e type);
static void _rec_state_changed_cb(recorder_state_e previous, recorder_state_e current, bool by_policy, void *user_data);
static void _recorder_recording_limit_reached_cb(recorder_recording_limit_type_e type, void *user_data);
static void _voice_memo_set_filename_to_recorder(char *filename);

static char *_voice_memo_get_filepath_from_recorder(void);
static void _voice_list_item_clicked_cb(void *user_data, Evas_Object *obj, void *event_info);



/*
 * @brief: Function will be operated when Main View is removed from naviframe
 * @param[user_data]: Data needed in this function
 * @param[it]: Item object of the main view
 * This means that this app will be terminated
 */
static Eina_Bool _naviframe_main_view_pop_cb(void *user_data, Elm_Object_Item *it)
{
	/* Exit from application */
	ui_app_exit();

	return EINA_FALSE;
}

/*
 * @brief: Function will be operated when Record View is removed from naviframe
 * @param[user_data]: Data passed to the function
 * @param[it]: Item object of the main view
 * This means that this app will be terminated
 */
static Eina_Bool _naviframe_record_view_pop_cb(void *user_data, Elm_Object_Item *it)
{
	/* Cancel the recording */
	if (!_voice_memo_control_recorder(CANCEL)) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Failed to cancel record");
		return EINA_FALSE;
	}

	if (s_info.pause_effect_timer) {
		ecore_timer_del(s_info.pause_effect_timer);
		s_info.pause_effect_timer = NULL;
		s_info.effect_count = 0;
	}

	s_info.record_view = NULL;

	return EINA_TRUE;
}

/*
 * @brief: Function will be operated when detail view is popped.
 * @param[user_data]: Data passed to the function
 * @param[it]: Elm Object Item of detail view
 * In this function, we should pop not only detail view but also record view
 * to go main view directly
 */
static Eina_Bool _detail_view_pop_cb(void *user_data, Elm_Object_Item *it)
{
	/*
	 * If there are some other item between main view and current view,
	 * they will be popped and destroyed
	 */
	elm_naviframe_item_pop_to(s_info.main_view_item);

	return EINA_TRUE;
}

/*
 * @brief: Create 'Record View' and start to record
 * @param[user_data]: Data passed to the function
 * @param[obj]: Main View
 * @param[event_info]: The pointer to an event specific, structure or information
 *                     to pass to the callback functions registered on this smart event
 */
static void _main_clicked_cb(void *user_data, Evas_Object *obj, void *event_info)
{
	char *file_title = NULL;
	char file_path[PATH_MAX] = { 0, };

	data_get_resource(EDJ_FILE, file_path, (int) PATH_MAX);

	/* File title means index of this recording */
	file_title = data_get_recording_file_title_according_to_index();

	/*
	 *  We'll set Record view's frame using EDJ file especially with 'record.view' group in EDJ
	 */
	s_info.record_view = view_create_layout(view_get_win(), file_path, "record.view", NULL, NULL);
	view_set_text(s_info.record_view, "rec.view.title", file_title);
	view_set_text(s_info.record_view, "rec.view.time", "00:00");

	/*
	 * Create record button with the function will be called on click
	 */
	view_set_button(s_info.record_view, "rec.view.btn", "focus", NULL, NULL, NULL, NULL, _rec_stop_clicked_cb, NULL);

	/*
	 * Create cancel button with the function will be called on click
	 */
	view_set_button(s_info.record_view, "rec.view.cancel", "focus", NULL, NULL, NULL, NULL, _rec_cancel_clicked_cb, NULL);

	/*
	 * Create pause button with the function will be called on click
	 */
	view_set_button(s_info.record_view, "rec.view.pause", "focus", NULL, NULL, NULL, NULL, _rec_pause_clicked_cb, s_info.record_view);

	s_info.record_view_item = view_push_item_to_naviframe(view_get_naviframe(), s_info.record_view, _naviframe_record_view_pop_cb, NULL);

	/*
	 * To start recording, it is necessary to set file name to the recorder
	 */
	char *file_name = data_get_filename_according_to_count();
	_voice_memo_set_filename_to_recorder(file_name);

	/*
	 * Start recording
	 */
	if (!_voice_memo_control_recorder(START)) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Failed to start record");
		view_pop_item_from_naviframe();
	}

	/*
	 * data_get_filename_according_to_count() function return with the memory, please free
	 */
	free(file_name);
	/*
	 * data_get_recording_file_title() function return with the memory, please free
	 */
	free(file_title);
}

/*
 * @brief: Function will be operated when record stop button is clicked
 * @param[user_data]: Data passed to the function
 * @param[obj]: Record view
 * @param[event_info]: The pointer to an event specific structure or information
 *                     to pass to the callback functions registered on this smart event
 */
static void _rec_stop_clicked_cb(void *user_data, Evas_Object *obj, void *event_info)
{
	Evas_Object *detail_view = NULL;
	Evas_Object *detail_more = NULL;
	voice_list_item_info *record_info = NULL;
	char file_path[PATH_MAX] = { 0, };
	char image_path[PATH_MAX] = { 0, };
	char *data_path = NULL;
	int i;

	/*
	 * Pause effect timer should be stop and initialized
	 */
	if (s_info.pause_effect_timer) {
		ecore_timer_del(s_info.pause_effect_timer);
		s_info.pause_effect_timer = NULL;
		s_info.effect_count = 0;
	}

	/*
	 * Save the record file
	 */
	if (!_voice_memo_control_recorder(SAVE)) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Failed to save a recorded file");
		view_pop_item_from_naviframe();
		return;
	}

	/*
	 * Get the information of recorded file and,
	 * convert to structure of voice memo list item info,
	 * to make management easy
	 */
	data_path = _voice_memo_get_filepath_from_recorder();
	record_info = data_get_recording_info_from_file_path(NULL, data_path);

	/*
	 *  We'll set Detail view's frame using EDJ file especially with 'detail.view' group in EDJ
	 */
	data_get_resource(EDJ_FILE, file_path, sizeof(file_path));
	detail_view = view_create_layout(view_get_win(), file_path, "detail.view", NULL, NULL);
	view_set_text(detail_view, "detail.view.title", record_info->title);
	view_set_text(detail_view, "detail.view.date", record_info->detail_date);
	view_set_text(detail_view, "detail.view.time", record_info->detail_time);
	view_set_button(detail_view, "detail.view.play", "bottom", NULL, NULL, NULL, NULL, _detail_play_clicked_cb, NULL);

	detail_more = view_set_more_button(detail_view, "detail.view.more", _detail_more_item_clicked_cb, NULL);
	for (i = 0; i < DETAIL_MORE_ITEM_MAX; i++) {
		data_get_resource(detail_more_item[i].icon, image_path, sizeof(image_path));
		view_create_more_item(detail_more, i, detail_more_item[i].name, detail_more_item[i].sub_name, image_path);
	}

	/*
	 * Add this voice memo to voice memo list
	 */
	data_append_item_info_to_list(record_info);
	/*
	 * Save this voice memo to check what file is selected
	 */
	data_set_selected_item(record_info);
	/*
	 * Increase index for next voice memo
	 */
	data_increase_index();

	view_push_item_to_naviframe(view_get_naviframe(), detail_view, _detail_view_pop_cb, NULL);
}

/*
 * @brief: Function will be operated when record cancel button is clicked
 * @param[user_data]: Data passed to the function
 * @param[obj]: Record view
 * @param[event_info]: The pointer to an event specific structure or information
 *                     to pass to the callback functions registered on this smart event
 */
static void _rec_cancel_clicked_cb(void *user_data, Evas_Object *obj, void *event_info)
{
	/* Pause effect should be stop and initialized */
	if (s_info.pause_effect_timer) {
		ecore_timer_del(s_info.pause_effect_timer);
		s_info.pause_effect_timer = NULL;
		s_info.effect_count = 0;
	}

	/* Cancel recording */
	if (!_voice_memo_control_recorder(CANCEL)) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Failed to cancel record");
	}

	/* Pop & destroy record view to go to main view */
	view_pop_item_from_naviframe();
}

/*
 * @brief: Send signal to EDJ to change image on every 0.5 second to show pause effect
 * @param[user_data]: Data passed to the function
 * As changing the state of the EDJ part, it can be shown like flickering
 */
static Eina_Bool _pause_effect_timer_cb(void *user_data)
{
	Evas_Object *record_view = user_data;
	s_info.effect_count++;

	if (s_info.effect_count % 2) {
		/* Send signal to make time text dark */
		view_send_signal_to_edje(record_view, "flickering", "dark");
	} else {
		/* Send signal to make time text bright */
		view_send_signal_to_edje(record_view, "flickering", "bright");
	}

	/* This return means that this function will be called repeatedly */
	return ECORE_CALLBACK_RENEW;
}

/*
 * @brief: Function will be operated when record pause button is clicked
 * @param[user_data]: Data passed to the function
 * @param[obj]: Record view
 * @param[event_info]: Some information regarding with this click event
 */
static void _rec_pause_clicked_cb(void *user_data, Evas_Object *obj, void *event_info)
{
	Evas_Object *record_view = user_data;
	recorder_state_e state;
	int err;

	err = recorder_get_state(s_info.recorder, &state);
	if (err != RECORDER_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "err = %d", err);
	}

	if (state != RECORDER_STATE_PAUSED) {
		/*
		 * Make recorder pause
		 */
		if (!_voice_memo_control_recorder(PAUSE)) {
			dlog_print(DLOG_ERROR, LOG_TAG, "Failed to pause record");
			view_pop_item_from_naviframe();
			return;
		}

		/*
		 * state is not paused now, change to pause
		 * to change the icon to pause, send signal to EDJ
		 */
		view_send_signal_to_edje(record_view, "pause", "pause");

		/* Using timer, show flickering effect on recording time text */
		s_info.pause_effect_timer = ecore_timer_add(0.5f, _pause_effect_timer_cb, record_view);

	} else {
		if (!_voice_memo_control_recorder(START)) {
			dlog_print(DLOG_ERROR, LOG_TAG, "Failed to start record");
			view_pop_item_from_naviframe();
			return;
		}

		view_send_signal_to_edje(record_view, "record", "record");

		/* Send signal to make recording time text not paused */
		view_send_signal_to_edje(record_view, "flickering", "bright");

		/* Delete the effect of pause and initialize for the next use */
		if (s_info.pause_effect_timer) {
			ecore_timer_del(s_info.pause_effect_timer);
			s_info.pause_effect_timer = NULL;
			s_info.effect_count = 0;
		}
	}
}

/*
 * @brief: Get the file path using recorder API
 */
static char *_voice_memo_get_filepath_from_recorder(void)
{
	char *filepath = NULL;

	recorder_get_filename(s_info.recorder, &filepath);
	return strdup(filepath);
}

/*
 * @brief: Function will be operated when more option item is clicked
 * @param[user_data]: Data passed to the function
 * @param[obj]: Main view
 * @param[event_info]: Clicked item in more option
 */
static void _main_more_item_clicked_cb(void *user_data, Evas_Object *obj, void *event_info)
{
	Eext_Object_Item *item = (Eext_Object_Item *)event_info;
	Evas_Object *voice_list_genlist = NULL;
	Eina_List *l;
	voice_list_item_info *list_data;
	int item_count = 0;
	const char *main_text;
	const char *sub_text;

	/* you can get more option item's text using this function */
	main_text = eext_more_option_item_part_text_get(item, "selector,main_text");
	sub_text = eext_more_option_item_part_text_get(item, "selector,sub_text");

	/* if selected item is 'List', make for voice memo list */
	if (!strcmp(main_text, "List")) {
		char image_path[PATH_MAX] = { 0, };

		data_get_resource("image/b_icon_voicememo.png", image_path, sizeof(image_path));

		item_count = data_get_voice_list_item_count();

		/* if there's no voice memo, show no voice memo view */
		if (item_count == 0) {
			Evas_Object *novoice_view = NULL;

			novoice_view = view_create_layout_no_content(view_get_naviframe(), "Recordings", "No recordings", image_path);
		} else {
			/* From here, we make genlist for voice memo list */
			/* make genlist and shape genlist circle */
			voice_list_genlist = view_create_circle_genlist(view_get_naviframe());

			view_append_item_to_genlist(voice_list_genlist, "menu.title", NULL, NULL, NULL);

			/* Get the list of saved voice memo */
			Eina_List *voice_list = data_get_voice_list();
			/* Append voice memo to the genlist */
			EINA_LIST_FOREACH(voice_list, l, list_data) {
				view_append_item_to_genlist(voice_list_genlist, list_data->style, list_data, _voice_list_item_clicked_cb, NULL);
			}

			view_append_item_to_genlist(voice_list_genlist, "padding", NULL, NULL, NULL);
		}

		s_info.genlist_item = view_push_item_to_naviframe(view_get_naviframe(), voice_list_genlist, NULL, NULL);
	}
}

/*
 * @brief: Function will be operated when more option item is clicked
 * @param[user_data]: Data passed to the function
 * @param[obj]: More Option object
 * @param[event_info]: Clicked item in More Option
 * In this sample, this function is for deletion
 */
static void _detail_more_item_clicked_cb(void *user_data, Evas_Object *obj, void *event_info)
{
	int voice_list_count = 0;

	/*
	 * Search the voice memo from voice memo list which voice memo is match with selected voice memo
	 * This search the list from start to end of the list
	 */
	data_remove_selected_item_from_list_and_db();
	view_pop_item_from_naviframe();

	/* If there's no voice memo */
	voice_list_count = data_count_voice_list_item();
	if (voice_list_count == 0) {
		Evas_Object *novoice_view = NULL;
		char image_path[PATH_MAX] = { 0, };

		/* Delete genlist used for voice memo list */
		view_destroy_object_item(s_info.genlist_item);

		data_get_resource("image/b_icon_voicememo.png", image_path, sizeof(image_path));
		/*
		 * Create 'No item view', you can customize second param for title,
		 * third param for detail text and forth param for image
		 */
		novoice_view = view_create_layout_no_content(view_get_naviframe(), "Recordings", "No recordings", image_path);
	}
}

/*
 * @brief: Function will be operated when the voice list memo is clicked
 * @param[user_data]: Data passed to the function
 * @param[obj]: Genlist has several items
 * @param[event_info]: Elm_Object_Item of the selected voice memo
 */
static void _voice_list_item_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Object *detail_view = NULL;
	Evas_Object *detail_more = NULL;
	voice_list_item_info *item_info = NULL;
	Elm_Object_Item *item = (Elm_Object_Item *) event_info;
	item_info = (voice_list_item_info *)elm_object_item_data_get(item);
	char file_path[PATH_MAX] = { 0, };
	char image_path[PATH_MAX] = { 0, };
	int i;

	/* Save the Elm_Object_Item for deletion this from the genlist or naviframe */
	item_info->item = item;

	data_get_resource(EDJ_FILE, file_path, sizeof(file_path));
	detail_view = view_create_layout(view_get_naviframe(), file_path, "detail.view", NULL, NULL);
	view_set_text(detail_view, "detail.view.title", item_info->title);
	view_set_text(detail_view, "detail.view.date", item_info->detail_date);
	view_set_text(detail_view, "detail.view.time", item_info->detail_time);
	view_set_button(detail_view, "detail.view.play", "bottom", NULL, NULL, NULL, NULL, _detail_play_clicked_cb, NULL);

	detail_more = view_set_more_button(detail_view, "detail.view.more", _detail_more_item_clicked_cb, NULL);
	for (i = 0; i < DETAIL_MORE_ITEM_MAX; i++) {
		data_get_resource(detail_more_item[i].icon, image_path, sizeof(image_path));
		view_create_more_item(detail_more, i, detail_more_item[i].name, detail_more_item[i].sub_name, image_path);
	}
	data_set_selected_item(item_info);

	view_push_item_to_naviframe(view_get_naviframe(), detail_view, NULL, NULL);

};

/*
 * @brief: Function will be operated when play button on detail view is clicked
 * @param[user_data]: Data passed to the function
 * @param[obj]: Detail view
 * @param[event_info]: Some information regarding with this click event
 */
static void _detail_play_clicked_cb(void *user_data, Evas_Object *obj, void *event_info)
{
	/* Do what do you want to do when play button is clicked */
}

/*
 * @brief: Function will be called when the status of the recorder is changed
 * @param[elapsed_time] The time of the recording (milliseconds)
 * @param[file_size] The size of the recording file (KB)
 * @param[user_data] The user data passed from the callback registration function
 */
static void _recording_status_cb(unsigned long long elapsed_time, unsigned long long file_size, void *user_data)
{
	int min = 0;
	int sec = 0;
	static int time = 0;
	char recording_time[256];

	sec = (int)(elapsed_time / 1000) % PER_SEC;

	if (time == sec) {
		return;
	}
	time = sec;

	min = (int)(elapsed_time / 1000) / PER_MIN;
	snprintf(recording_time, sizeof(recording_time), "%02i:%02i", min, sec);

	/* Set the time to 'rec.view.time' part */
	view_set_text(s_info.record_view, "rec.view.time", recording_time);
}

/*
 * @brief: Create recorder handle, and initialize several attributes for recorder
 */
static void _voice_memo_initialize_recorder(void)
{
	int err;

	/* Creates a recorder handle to record an audio */
	err = recorder_create_audiorecorder(&s_info.recorder);
	if (err != RECORDER_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "err = %d", err);
	}

	/* Sets the function will be called when the recorder state is changed */
	err = recorder_set_state_changed_cb(s_info.recorder, _rec_state_changed_cb, NULL);
	if (err != RECORDER_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "err = %d", err);
	}

	/* Sets the audio codec for encoding an audio stream */
	err = recorder_set_audio_encoder(s_info.recorder, RECORDER_AUDIO_CODEC_AAC);
	if (err != RECORDER_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "err = %d", err);
	}

	/* Sets the file format for recording media stream */
	err = recorder_set_file_format(s_info.recorder, RECORDER_FILE_FORMAT_3GP);
	if (err != RECORDER_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "err = %d", err);
	}

	/* Sets the time limit of a recording file */
	err = recorder_attr_set_time_limit(s_info.recorder, RECORDING_TIME_LIMIT);
	if (err != RECORDER_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "err = %d", err);
	}

	/* Sets the bit rate of a recording file */
	err = recorder_attr_set_audio_encoder_bitrate(s_info.recorder, 28800);
	if (err != RECORDER_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "err = %d", err);
	}

	/* Set audio device to microphone */
	err = recorder_attr_set_audio_device(s_info.recorder, RECORDER_AUDIO_DEVICE_MIC);
	if (err != RECORDER_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "err = %d", err);
	}

	/* Set audio sample rate */
	err = recorder_attr_set_audio_samplerate(s_info.recorder, 44100);
	if (err != RECORDER_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "err = %d", err);
	}

	/*
	 * Registers a callback function to be invoked when the recording information changes.
	 */
	err = recorder_set_recording_status_cb(s_info.recorder, _recording_status_cb, NULL);
	if (err != RECORDER_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "err = %d", err);
	}

	/*
	 * Registers the callback function to be run when reached the recording limit
	 * Customize '_recorder_recording_limit_reached_cb' to do what you want to do
	 */
	err = recorder_set_recording_limit_reached_cb(s_info.recorder, _recorder_recording_limit_reached_cb, NULL);
	if (err != RECORDER_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "err = %d", err);
	}
}

/*
 * @brief: Deinitialize several attributes for recorder and destroy the recorder handle
 */
static void _voice_memo_deinitialize_recorder(void)
{
	int err;

	err = recorder_unset_state_changed_cb(s_info.recorder);
	if (err != RECORDER_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "err = %d", err);
	}
	err = recorder_unprepare(s_info.recorder);
	if (err != RECORDER_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "err = %d", err);
	}

	err = recorder_unset_recording_limit_reached_cb(s_info.recorder);
	if (err != RECORDER_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "err = %d", err);
	}

	err = recorder_unset_recording_status_cb(s_info.recorder);
	if (err != RECORDER_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "err = %d", err);
	}

	err = recorder_destroy(s_info.recorder);
	if (err != RECORDER_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "err = %d", err);
	}
}

/*
 * @brief: Function will be called when the state of the recorder is changed
 * @param[previous]: Previous state of the recorder
 * @param[current]: Current state of the recorder
 * @param[by_policy]: True if the state is changed by policy, otherwise false if the state is not changed
 * @param[user_data]: The user data passed from the callback registration function
 */
static void _rec_state_changed_cb(recorder_state_e previous, recorder_state_e current, bool by_policy, void *user_data)
{
	dlog_print(DLOG_DEBUG, LOG_TAG , "_recorder_state_changed_cb (prev: %d, curr: %d)\n", previous, current);
}

/*
 * @brief: Function will be called when the recorder is reached to limit on the type you've set
 * @param[type]: Three types are available, TIME, SIZE or No free space
 * @param[user_data]: The user data passed from the callback registration function
 */
static void _recorder_recording_limit_reached_cb(recorder_recording_limit_type_e type, void *user_data)
{
	dlog_print(DLOG_DEBUG, LOG_TAG , "Recording limit reached: %d\n", type);
}

/*
 * @brief: Set file name to start recording
 * If same file name is set, it will be over written
 */
static void _voice_memo_set_filename_to_recorder(char *filename)
{
	int err;

	err = recorder_set_filename(s_info.recorder, filename);
	if (err != RECORDER_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "err = %d", err);
	}
}


/*
 * @brief: Manage several record function
 * @param[type]: Operation type, indicate what will be done with the recorder
 */
static Eina_Bool _voice_memo_control_recorder(recorder_control_type_e type)
{
	int err;
	recorder_state_e state;

	err = recorder_get_state(s_info.recorder, &state);
	if (err != RECORDER_ERROR_NONE) {
	}

	switch (type) {
	case PREPARE:
		/* Prepare is available only on RECORDER_STATE_CREATED */
		if (state  == RECORDER_STATE_CREATED) {
			err = recorder_prepare(s_info.recorder);
			if (err != RECORDER_ERROR_NONE) {
				Evas_Object *layout = view_get_base_layout();
				char *popup_text = data_get_popup_text();
				view_create_text_popup(layout, 2.0, popup_text);
				free(popup_text);

				dlog_print(DLOG_ERROR, LOG_TAG, "PREPARE err = %d", err);

				return EINA_FALSE;
			}
		}
		break;
	case START:
		/* Start is available only on RECORDER_STATE_READY or PAUSED */
		if (state  == RECORDER_STATE_READY || state == RECORDER_STATE_PAUSED) {
			err = recorder_start(s_info.recorder);
			if (err != RECORDER_ERROR_NONE) {
				Evas_Object *layout = view_get_base_layout();
				char *popup_text = data_get_popup_text();
				view_create_text_popup(layout, 2.0, popup_text);
				free(popup_text);

				dlog_print(DLOG_ERROR, LOG_TAG, "START err = %d", err);

				if (state == RECORDER_STATE_PAUSED) {
					err = recorder_cancel(s_info.recorder);
					if (err != RECORDER_ERROR_NONE) {
						dlog_print(DLOG_ERROR, LOG_TAG, "CANCEL err = %d", err);
						return EINA_FALSE;
					}
				}

				return EINA_FALSE;
			}
		}
		break;
	case PAUSE:
		/* Pause is available only on RECORDER_STATE_RECORDING */
		if (state  == RECORDER_STATE_RECORDING) {
			err = recorder_pause(s_info.recorder);
			if (err != RECORDER_ERROR_NONE) {
				Evas_Object *layout = view_get_base_layout();
				char *popup_text = data_get_popup_text();
				view_create_text_popup(layout, 2.0, popup_text);
				free(popup_text);

				dlog_print(DLOG_ERROR, LOG_TAG, "PAUSE err = %d", err);

				err = recorder_cancel(s_info.recorder);
				if (err != RECORDER_ERROR_NONE) {
					dlog_print(DLOG_ERROR, LOG_TAG, "CANCEL err = %d", err);
					return EINA_FALSE;
				}

				return EINA_FALSE;
			}
		}
		break;
	case CANCEL:
		/* Cancel is available only on RECORDER_STATE_RECORDING or PAUSED */
		if (state  == RECORDER_STATE_RECORDING || state == RECORDER_STATE_PAUSED) {
			err = recorder_cancel(s_info.recorder);
			if (err != RECORDER_ERROR_NONE) {
				Evas_Object *layout = view_get_base_layout();
				char *popup_text = data_get_popup_text();
				view_create_text_popup(layout, 2.0, popup_text);
				free(popup_text);

				dlog_print(DLOG_ERROR, LOG_TAG, "CANCEL err = %d", err);

				return EINA_FALSE;
			}
		}
		break;
	case SAVE:
		/* Save is available only on RECORDER_STATE_RECORDING or PAUSED */
		if (state == RECORDER_STATE_RECORDING || state == RECORDER_STATE_PAUSED) {
			err = recorder_commit(s_info.recorder);
			if (err != RECORDER_ERROR_NONE) {
				Evas_Object *layout = view_get_base_layout();
				char *popup_text = data_get_popup_text();
				view_create_text_popup(layout, 2.0, popup_text);
				free(popup_text);

				dlog_print(DLOG_ERROR, LOG_TAG, "SAVE err = %d", err);

				err = recorder_cancel(s_info.recorder);
				if (err != RECORDER_ERROR_NONE) {
					dlog_print(DLOG_ERROR, LOG_TAG, "CANCEL err = %d", err);
					return EINA_FALSE;
				}

				return EINA_FALSE;
			}
		}
		break;
	default:
		break;
	}

	return EINA_TRUE;
}

/*
 * @brief: Hook to take necessary actions before main event loop starts
 * Initialize UI resources and application's data
 * If this function returns true, the main loop of application starts
 * If this function returns false, the application is terminated
 */
static bool app_create(void *user_data)
{
	char file_path[PATH_MAX] = { 0, };
	char image_path[PATH_MAX] = { 0, };
	int i;
	Evas_Object *main_more = NULL;
	Evas_Object *main_view = NULL;

	/*
	 * Create Essential component window, conformant and layout
	 */
	view_create();

	view_voice_memo_create();

	/*
	 * For recording, there are something to initialize in this function
	 */
	_voice_memo_initialize_recorder();

	/*
	 * Making recorder state ready, it is necessary to start recording
	 */
	if (!_voice_memo_control_recorder(PREPARE)) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Failed to prepare record");
		view_pop_item_from_naviframe();
		return false;
	}

	/*
	 * Check the index of the voice memo will be saved next
	 */
	data_get_index();
	/*
	 * To manage(show, delete, add) voice memo easily,
	 * Get voice memo files from the DB and convert it to the Structure
	 */
	data_create_voice_list_from_db();


	/*
	 * Start to create 'main view' shown at first when the application is launched
	 * We'll set Main view's frame using EDJ file especially with 'main.view' group in EDJ
	 */
	data_get_resource(EDJ_FILE, file_path, sizeof(file_path));
	main_view = view_create_layout(view_get_win(), file_path, "main.view", NULL, NULL);
	/*
	 * Set the text to 'main.view.title' part of 'main.view' group in EDJ file
	 */
	view_set_text(main_view, "main.view.title", "Voice Memo");
	view_set_text(main_view, "main.view.time", "00:00");
	/*
	 * Create Recording button
	 * Set the button to "main.view.rec" part with '_main_clicked_cb' function called on click
	 */
	view_set_button(main_view, "main.view.rec", "focus", NULL, NULL, NULL, NULL, _main_clicked_cb, NULL);
	/*
	 * Create 'More Option' button
	 * '_main_more_item_clicked_cb' function is called when 'More Option's item is clicked
	 */
	main_more = view_set_more_button(main_view, "main.view.more", _main_more_item_clicked_cb, NULL);
	/*
	 * Create 'More Option's item as many as you want
	 * Also can assign the name, sub text and the image of the item
	 */
	for (i = 0; i < MAIN_MORE_ITEM_MAX; ++i) {
		data_get_resource(main_more_item[i].icon, image_path, sizeof(image_path));
		view_create_more_item(main_more, i, main_more_item[i].name, main_more_item[i].sub_name, image_path);
	}

	/*
	 * After creating the 'Main View', push it to naviframe's first stack
	 * '_naviframe_main_view_pop_cb' is a function when 'Main View' is removed from the naviframe
	 */
	s_info.main_view_item = view_push_item_to_naviframe(view_get_naviframe(), main_view, _naviframe_main_view_pop_cb, NULL);

	return true;
}

/*
 * @brief: This callback function is called when another application
 * sends the launch request to the application
 */
static void app_control(app_control_h app_control, void *user_data)
{
	/* Handle the launch request. */
}

/*
 * @brief: This callback function is called each time
 * the application is completely obscured by another application
 * and becomes invisible to the user
 */
static void app_pause(void *user_data)
{
	/* Take necessary actions when application becomes invisible. */
}

/*
 * @brief: This callback function is called each time
 * the application becomes visible to the user
 */
static void app_resume(void *user_data)
{
	/* Take necessary actions when application becomes visible. */
}

/*
 * @brief: This callback function is called once after the main loop of the application exits
 */
static void app_terminate(void *user_data)
{
	/* Release all resources. */

	/* Deinitialize recorder before the exit */
	_voice_memo_deinitialize_recorder();

	/* Release allocated voice info in the voice list */
	data_release_voice_list();

	/* Store saved memo index for next launch */
	data_store_index();

	/* Release window resources */
	view_destroy();
}

/*
 * @brief: This function will be called when the language is changed
 */
static void ui_app_lang_changed(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_LANGUAGE_CHANGED*/
	char *locale = NULL;

	system_settings_get_value_string(SYSTEM_SETTINGS_KEY_LOCALE_LANGUAGE, &locale);

	if (locale != NULL) {
		elm_language_set(locale);
		free(locale);
	}
	return;
}

/*
 * @brief: Main function of the application
 */
int main(int argc, char *argv[])
{
	int ret;

	ui_app_lifecycle_callback_s event_callback = {0, };
	app_event_handler_h handlers[5] = {NULL, };

	event_callback.create = app_create;
	event_callback.terminate = app_terminate;
	event_callback.pause = app_pause;
	event_callback.resume = app_resume;
	event_callback.app_control = app_control;

	/*
	 * If you want to handling more events,
	 * Please check the application lifecycle guide
	 */
	ui_app_add_event_handler(&handlers[APP_EVENT_LANGUAGE_CHANGED], APP_EVENT_LANGUAGE_CHANGED, ui_app_lang_changed, NULL);

	ret = ui_app_main(argc, argv, &event_callback, NULL);
	if (ret != APP_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "ui_app_main() is failed. err = %d", ret);
	}

	return ret;
}

