/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Elementary.h>
#include <efl_extension.h>
#include <app.h>
#include <dlog.h>

#include "$(appName).h"
#include "view.h"

#define PACKAGE "org.example.$(appName)"

static struct view_info {
	Evas_Object *win;
	Evas_Object *conform;
	Evas_Object *nf;
	Evas_Object *layout;
	Eext_Circle_Surface *circle_surface;
} s_info = {
	.win = NULL,
	.conform = NULL,
	.nf = NULL,
	.layout = NULL,
	.circle_surface = NULL,
};

static void _win_delete_request_cb(void *user_data, Evas_Object *obj, void *event_info);
static void _gl_selected_cb(void *user_data, Evas_Object *obj, void *event_info);
static char *_get_menu_title_text(void *user_data, Evas_Object *obj, const char *part_name);
static char *_get_item_text(void *user_data, Evas_Object *obj, const char *part_name);
static Elm_Genlist_Item_Class *_set_genlist_item_class(const char *style);

/*
 * @brief: Get window
 */
Evas_Object *view_get_win(void)
{
	return s_info.win;
}

/*
 * @brief: Get naviframe
 */
Evas_Object *view_get_naviframe(void)
{
	return s_info.nf;
}

/*
 * @brief: Get base layout
 */
Evas_Object *view_get_base_layout(void)
{
	return s_info.layout;
}

/*
 * @brief: Create Essential Object window, conformant and layout
 */
void view_create(void)
{
	/* Create window */
	s_info.win = view_create_win(PACKAGE);
	if (s_info.win == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to create a window.");
		return;
	}

	/* Create conformant */
	s_info.conform = view_create_conformant_without_indicator(s_info.win);
	if (s_info.conform == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to create a conformant");
		return;
	}

	/* Show window after main view is set up */
	evas_object_show(s_info.win);
}

/*
 * @brief: Make a basic window named package
 * @param[pkg_name]: Name of the window
 */
Evas_Object *view_create_win(const char *pkg_name)
{
	Evas_Object *win = NULL;

	/*
	 * Window
	 * Create and initialize elm_win.
	 * elm_win is mandatory to manipulate window.
	 */
	win = elm_win_util_standard_add(pkg_name, pkg_name);
	elm_win_conformant_set(win, EINA_TRUE);
	elm_win_autodel_set(win, EINA_TRUE);

	/* Rotation setting */
	if (elm_win_wm_rotation_supported_get(win)) {
		int rots[4] = { 0, 90, 180, 270 };
		elm_win_wm_rotation_available_rotations_set(win, (const int *)(&rots), 4);
	}

	evas_object_smart_callback_add(win, "delete,request", _win_delete_request_cb, NULL);

	return win;
}

/*
 * @brief: Make conformant without indicator for wearable app
 * @param[win]: Object to which you want to set this conformant
 * Conformant is mandatory for base GUI to have proper size
 */
Evas_Object *view_create_conformant_without_indicator(Evas_Object *win)
{
	/*
	 * Conformant
	 * Create and initialize elm_conformant.
	 * elm_conformant is mandatory for base GUI to have proper size
	 * when indicator or virtual keypad is visible.
	 */
	Evas_Object *conform = NULL;

	if (win == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "window is NULL.");
		return NULL;
	}

	conform = elm_conformant_add(win);
	evas_object_size_hint_weight_set(conform, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_win_resize_object_add(win, conform);

	evas_object_show(conform);

	return conform;
}

/*
 * @brief: Make a layout to target parent object with edje file
 * @param[parent]: The object to which you want to add this layout
 * @param[file_path]: File path of EDJ file will be used
 * @param[group_name]: Name of group in EDJ you want to set to
 * @param[cb_function]: The function will be called when back event is detected
 * @param[user_data]: The user data to be passed to the callback functions
 */
Evas_Object *view_create_layout(Evas_Object *parent, const char *file_path, const char *group_name, Eext_Event_Cb cb_function, void *user_data)
{
	Evas_Object *layout = NULL;

	if (parent == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "parent is NULL.");
		return NULL;
	}

	/* Create layout by EDC(edje file) */
	layout = elm_layout_add(parent);
	elm_layout_file_set(layout, file_path, group_name);

	/* Layout size setting */
	evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

	if (cb_function)
		eext_object_event_callback_add(layout, EEXT_CALLBACK_BACK, cb_function, user_data);

	evas_object_show(layout);

	return layout;
}

/*
 * @brief: Make a layout with theme.
 * @param[parent]: Object to which you want to add this layout
 * @param[class_name]: The class of the group
 * @param[group_name]: Group name in EDJ that you want to set to layout
 * @param[style]: The style to use
 */
Evas_Object *view_create_layout_by_theme(Evas_Object *parent, const char *class_name, const char *group_name, const char *style)
{
	/*
	 * Layout
	 * Create and initialize elm_layout.
	 * view_create_layout_by_theme() is used to create layout by using premade edje file.
	 */
	Evas_Object *layout = NULL;

	if (parent == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "parent is NULL.");
		return NULL;
	}

	layout = elm_layout_add(parent);
	elm_layout_theme_set(layout, class_name, group_name, style);
	evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

	evas_object_show(layout);

	return layout;
}

/*
 * @brief: Destroy window and free important data to finish this application
 */
void view_destroy(void)
{
	if (s_info.win == NULL) {
		return;
	}

	evas_object_del(s_info.win);
}

/*
 * @brief: Destroy given layout
 * @param[layout]: Layout you want to destroy
 */
void view_destroy_layout(Evas_Object *layout)
{
	evas_object_del(layout);
}

/*
 * @brief: Set image to given part
 * @param[parent]: Object has part to which you want to set this image
 * @param[part_name]: Part name to which you want to set this image
 * @param[image_path]: Path of the image file
 */
void view_set_image(Evas_Object *parent, const char *part_name, const char *image_path)
{
	Evas_Object *image = NULL;

	if (parent == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "parent is NULL.");
		return;
	}

	image = elm_object_part_content_get(parent, part_name);
	if (image == NULL) {
		image = elm_image_add(parent);
		if (image == NULL) {
			dlog_print(DLOG_ERROR, LOG_TAG, "failed to create an image object.");
			return;
		}

		elm_object_part_content_set(parent, part_name, image);
	}

	if (EINA_FALSE == elm_image_file_set(image, image_path, NULL)) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to set image.");
		return;
	}

	evas_object_show(image);

	return;
}

/*
 * @brief: Set text to the part
 * @param[parent]: Object has part to which you want to set text
 * @param[part_name]: Part name to which you want to set the text
 * @param[text]: Text you want to set to the part
 */
void view_set_text(Evas_Object *parent, const char *part_name, const char *text)
{
	if (parent == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "parent is NULL.");
		return;
	}

	/* Set text of target part object */
	elm_object_part_text_set(parent, part_name, text);
}

/*
 * @brief: Make a naviframe and set to parent
 * @param[parent]: Object to which you want to set naviframe
 * @Add callback function will be operated when back key is pressed
 */
void view_create_naviframe(Evas_Object *parent)
{
	/* Naviframe
	 * Create and initialize elm_naviframe.
	 * Naviframe stands for navigation frame.
	 * elm_naviframe is a views manager for applications.
	 */
	s_info.nf = elm_naviframe_add(parent);

	elm_object_part_content_set(parent, "elm.swallow.content", s_info.nf);
	eext_object_event_callback_add(s_info.nf, EEXT_CALLBACK_BACK, eext_naviframe_back_cb, NULL);
	eext_object_event_callback_add(s_info.nf, EEXT_CALLBACK_MORE, eext_naviframe_more_cb, NULL);

	evas_object_show(s_info.nf);
}

/*
 * @brief: Push item to naviframe
 * @param[nf]: Naviframe has several views
 * @param[item]: Object will be added to naviframe
 * @param[pop_cb]: Function will be operated when this item is popped from naviframe
 * @param[user_data]: Data passed to the 'pop_cb' function
 * Naviframe make changing of view is easy and effectively
 */
Elm_Object_Item* view_push_item_to_naviframe(Evas_Object *nf, Evas_Object *item, Elm_Naviframe_Item_Pop_Cb pop_cb, void *user_data)
{
	Elm_Object_Item* nf_it = NULL;

	if (nf == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "naviframe is NULL.");
		return NULL;
	}

	if (item == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "item is NULL.");
		return NULL;
	}

	nf_it = elm_naviframe_item_push(nf, NULL, NULL, NULL, item, "empty");

	if (pop_cb != NULL)
		elm_naviframe_item_pop_cb_set(nf_it, pop_cb, user_data);

	return nf_it;
}

/*
 * @brief: Make more option object
 * @param[parent]: Object to which you want to set the more option
 * @param[part_name]: Part name for more option, set part size as large as parent
 * @param[_item_selected]: Function will be operated when more option item is clicked
 * @param[user_data]: Data passed to the '_item_selected' function
 */
Evas_Object *view_set_more_button(Evas_Object *parent, const char *part_name, Evas_Smart_Cb _item_selected, void *user_data)
{
	Evas_Object *more_btn = NULL;

	if (parent == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "parent is NULL.");
		return NULL;
	}

	more_btn = eext_more_option_add(parent);
	if (more_btn == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "more option is NULL.");
		return NULL;
	}

	/* Add smart callback */
	evas_object_smart_callback_add(more_btn, "item,clicked", _item_selected, user_data);

	elm_object_part_content_set(parent, part_name, more_btn);

	return more_btn;
}

/*
 * @brief: Make and set button.
 * @param[parent]: Object to which you want to set the button
 * @param[part_name]: Name of part to which you want to set the button
 * @param[style]: Style of the button
 * @param[image_path]: Path of image file will be used as button icon
 * @param[text]: The text will be written on the button
 * @param[down_cb]: Function will be operated when the button is pressed
 * @param[up_cb]: Function will be operated when the button is released
 * @param[clicked_cb]: Function will be operated when the button is clicked
 * @param[user_data]: Data passed to the 'clicked_cb' function
 */
void view_set_button(Evas_Object *parent, const char *part_name, const char *style, const char *image_path, const char *text,
		Evas_Object_Event_Cb down_cb, Evas_Object_Event_Cb up_cb, Evas_Smart_Cb clicked_cb, void *user_data)
{
	Evas_Object *btn = NULL;

	if (parent == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "parent is NULL.");
		return;
	}

	btn = elm_button_add(parent);
	if (btn == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to create button.");
		return;
	}

	if (style)
		elm_object_style_set(btn, style);

	evas_object_size_hint_weight_set(btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_object_part_content_set(parent, part_name, btn);

	if (text)
		elm_object_text_set(btn, text);

	if (image_path)
		view_set_image(btn, NULL, image_path);

	if (down_cb)
		evas_object_event_callback_add(btn , EVAS_CALLBACK_MOUSE_DOWN, down_cb, user_data);
	if (up_cb)
		evas_object_event_callback_add(btn, EVAS_CALLBACK_MOUSE_UP, up_cb, user_data);
	if (clicked_cb)
		evas_object_smart_callback_add(btn, "clicked", clicked_cb, user_data);

	evas_object_show(btn);
}

/*
 * @brief: Make genlist and circle_genlist for circular shape
 * @param[parent]: Naviframe to which you want to set the genlist
 */
Evas_Object *view_create_circle_genlist(Evas_Object *parent)
{
	Evas_Object *genlist = NULL;
	Evas_Object *circle_genlist = NULL;

	if (parent == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "parent is NULL.");
		return NULL;
	}

	if (s_info.circle_surface == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "circle surface is NULL.");
		return NULL;
	}

	genlist = elm_genlist_add(parent);
	/* this make selected list item is shown compressed */
	elm_genlist_mode_set(genlist, ELM_LIST_COMPRESS);
	evas_object_smart_callback_add(genlist, "selected", _gl_selected_cb, NULL);

	/* this make genlist style circular */
	circle_genlist = eext_circle_object_genlist_add(genlist, s_info.circle_surface);
	eext_circle_object_genlist_scroller_policy_set(circle_genlist, ELM_SCROLLER_POLICY_OFF, ELM_SCROLLER_POLICY_AUTO);
	eext_rotary_object_event_activated_set(circle_genlist, EINA_TRUE);

	evas_object_show(genlist);

	return genlist;
}

/*
 * @brief: Add item to genlist.
 * @param[genlist]: Genlist shows several items
 * @param[style]: Style of item determine how to show this item, such as "1text", "1text1icon" and so on
 * @param[gen_data]: Item data that use item's callback function
 * @param[clicked_cb]: Function will be operated when the item is clicked
 * @param[user_data]: Data passed to 'clicked_cb' function
 * This make item's class and add item to genlist.
 */
Elm_Object_Item *view_append_item_to_genlist(Evas_Object *genlist, const char *style,
		const void *gen_data, Evas_Smart_Cb clicked_cb, const void *user_data)
{
	Elm_Genlist_Item_Class *item_class;
	Elm_Object_Item *item;

	if (genlist == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "genlist is NULL.");
		return NULL;
	}

	if (style == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "item style is NULL.");
		return NULL;
	}

	item_class = _set_genlist_item_class(style);

	item = elm_genlist_item_append(genlist, item_class, gen_data, NULL, ELM_GENLIST_ITEM_NONE, clicked_cb, user_data);

	elm_genlist_item_class_free(item_class);

	return item;
}

/*
 * @brief: Send signal with source to the EDJ object
 * @param[layout]: The layout object will receive the signal
 * @param[signal]: The appointed signal to trigger the function
 * @param[source]: The appointed source that normally indicate the object triggered the event
 */
void view_send_signal_to_edje(Evas_Object *layout, const char *signal, const char *source)
{
	elm_object_signal_emit(layout, signal, source);
}

/*
 * @brief: Make no content view
 * @param[parent]: Naviframe has several views
 * @param[title]: Title of this view
 * @param[detail]: Detail text of this view
 * @param[image_path]: Image file path will be used in this view
 * This function make a view consists of one title, one center image and one detail text at the bottom
 */
Evas_Object *view_create_layout_no_content(Evas_Object *parent, char *title, char *detail, char *image_path)
{
	Evas_Object *layout = NULL;
	Evas_Object *image;
	Elm_Object_Item *it;

	if (parent == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "parent is NULL.");
		return NULL;
	}

	layout = elm_layout_add(parent);
	/* this make you can use layout consists of one title, one image and one detail text */
	elm_layout_theme_set(layout, "layout", "nocontents", "default");
	evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(layout, EVAS_HINT_FILL, EVAS_HINT_FILL);

	if (image_path) {
		image = elm_image_add(parent);
		elm_image_file_set(image, image_path, NULL);
		elm_object_part_content_set(layout, "elm.swallow.icon", image);
		evas_object_show(image);
	}

	if (detail) {
		char detail_text[256] = {0, };

		/* Customize font style bellow */
		snprintf(detail_text, sizeof(detail_text), "<font=TIZEN:style=Medium align=center color=#FFFFFF><font_size=28>%s</font_size></font>", detail);
		elm_object_part_text_set(layout, "elm.text", detail_text);
	}
	if (title) {
		char title_text[256] = {0, };

		snprintf(title_text, sizeof(title_text), "<font=TIZEN:style=Medium align=center color=#12B4FF><font_size=28>%s</font_size></font>", title);
		elm_object_part_text_set(layout, "elm.text.title", title_text);
	}

	/* if this signal is not sent, title text will be disappear */
	elm_object_signal_emit(layout, "elm,state,title,enable", "elm");
	/* push this no voice memo view to naviframe to show */
	it = elm_naviframe_item_push(parent, "Title Enabled", NULL, NULL, layout, NULL);
	/* if this is EINA_TRUE, you can see Title of naviframe on the top of the screen */
	elm_naviframe_item_title_enabled_set(it, EINA_FALSE, EINA_FALSE);

	return layout;
}

/*
 * @brief: Create additional object for voice memo
 */
void view_voice_memo_create(void)
{
	/* This make this application can show circular layout for wearable */
	s_info.circle_surface = eext_circle_surface_conformant_add(s_info.conform);

	/* In this function, we make layout and set to conformant
	 * Second parameter is file name, third parameter is group name you want to set to */
	s_info.layout = view_create_layout_for_conformant_by_theme(s_info.conform, "layout", "application", "default");
	if (s_info.layout == NULL) {
		evas_object_del(s_info.win);
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to create a base_layout.");
		return;
	}
	/*
	 * To Manage several views easily, use naviframe
	 * Naviframe manage many views as a stack
	 */
	view_create_naviframe(s_info.layout);
}

/*
 * @brief: Delete genlist used for voice memo list
 * @param[item]: The item will be deleted
 */
void view_destroy_object_item(Elm_Object_Item *item)
{
	if (item != NULL) {
		elm_object_item_del(item);
	}
}

/*
 * @brief: Pop item to naviframe
 */
void view_pop_item_from_naviframe(void)
{
	elm_naviframe_item_pop(s_info.nf);
}

/*
 * @brief: Make more option item
 * @param[parent]: More option
 * @param[index]: Index of item
 * @param[name]: Name of item
 * @param[sub_name]: Detail text of item if it is needed
 * @param[image_path]: Image file name will be used as icon
 */
void view_create_more_item(Evas_Object *parent, int index, char *name, char *sub_name, char *image_path)
{
	Evas_Object *image = NULL;

	/* Create the new item */
	Eext_Object_Item *item  = eext_more_option_item_append(parent);

	/* Set the text in item text part */
	if (name != NULL) {
		eext_more_option_item_part_text_set(item, "selector,main_text", name);
	}
	if (sub_name != NULL) {
		eext_more_option_item_part_text_set(item, "selector,sub_text", sub_name);
	}

	image = elm_image_add(parent);
	/* Set the content in item content part */
	eext_more_option_item_part_content_set(item, "item,icon", image);
	elm_image_file_set(image, image_path, NULL);

	return;
}

/*
 * @brief: Make layout by given theme and set to conformant
 * @param[parent]: Object to which you want to add this layout
 * @param[class_name]: The class of the group
 * @param[group_name]: Group name in EDJ you want to set to layout
 * @param[style]: The style of the theme
 */
Evas_Object *view_create_layout_for_conformant_by_theme(Evas_Object *parent, const char *class_name, const char *group_name, const char *style)
{
	Evas_Object *layout = NULL;

	/* Set theme for layout */
	layout = view_create_layout_by_theme(parent, class_name, group_name, style);

	elm_object_content_set(parent, layout);

	return layout;
}

/*
 * @brief: Function will be operated when window is deleted
 * @param[user_data]: Data passed to this callback function, in this case NULL
 * @param[obj]: Window
 * @param[event_info]: Informations regarding with this event
 */
static void _win_delete_request_cb(void *user_data, Evas_Object *obj, void *event_info)
{
	/* finish the app lifecycle */
	ui_app_exit();
}

/*
 * @brief: function will be operated when genlist item is selected
 * @param[user_data]: Data needed in this function
 * @param[obj]: Genlist has the item
 * @param[event_info]: Selected item
 */
static void _gl_selected_cb(void *user_data, Evas_Object *obj, void *event_info)
{
	Elm_Object_Item *it = (Elm_Object_Item *)event_info;
	elm_genlist_item_selected_set(it, EINA_FALSE);
}

/*
 * @brief: Set functions will be operated when this item is shown on the screen according to the style
 * @param[style]: Style of item
 */
static Elm_Genlist_Item_Class *_set_genlist_item_class(const char *style)
{
	Elm_Genlist_Item_Class *item_class = NULL;

	if (style == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "style is NULL.");
		return NULL;
	}

	item_class = elm_genlist_item_class_new();

	/* If you want to add the item class of genlist, you should be add below */
	if (!strcmp(style, "menu.title")) {
		item_class->item_style = "title";
		/* this function will be operated when this item is shown on the screen to get the title */
		item_class->func.text_get = _get_menu_title_text;
	} else if (!strcmp(style, "display.title")) {
		item_class->item_style = "title";
	} else if (!strcmp(style, "1text")) {
		item_class->item_style = "1text";
	} else if (!strcmp(style, "1text.1icon")) {
		item_class->item_style = "1text.1icon";
	} else if (!strcmp(style, "1text.1icon.1")) {
		item_class->item_style = "1text.1icon.1";
	} else if (!strcmp(style, "2text")) {
		item_class->item_style = "2text";
	} else if (!strcmp(style, "3text")) {
		item_class->item_style = "3text";
		item_class->func.text_get = _get_item_text;
	} else if (!strcmp(style, "padding")) {
		/* "padding" style does nothing
		 * But it makes genlist item placed in the middle of the screen
		 */
	}

	return item_class;
}

/*
 * @brief: Function will be operated when the item is shown
 * @param[user_data]: Data passed from 'elm_genlist_item_append' as third parameter
 * @param[obj]: Genlist has several items
 * @param[part_name]: Name of one of the existing text parts in the pre-made EDJ file for the genlist item
 */
static char *_get_menu_title_text(void *data, Evas_Object *obj, const char *part_name)
{
	char buf[BUF_LEN];

	snprintf(buf, sizeof(buf), "%s", "Memo List");
	return strdup(buf);
}

/*
 * @brief: Function will be operated when the item is shown
 * @param[user_data]: Data passed from 'elm_genlist_item_append' as third parameter
 * @param[obj]: Genlist for displaying several items
 * @param[part_name]: Name of one of the existing text parts in the EDJ file for the genlist item
 */
static char *_get_item_text(void *user_data, Evas_Object *obj, const char *part_name)
{
	char buf[BUF_LEN];
	voice_list_item_info *item_info = (voice_list_item_info *)user_data;

	if (!strcmp(part_name, "elm.text")) {
		snprintf(buf, sizeof(buf), "%s", item_info->title);
		return strdup(buf);
	} else if (!strcmp(part_name, "elm.text.1")) {
		snprintf(buf, sizeof(buf), "%s", item_info->date);
		return strdup(buf);
	} else if (!strcmp(part_name, "elm.text.2")) {
		snprintf(buf, sizeof(buf), "%s", item_info->duration);
		return strdup(buf);
	}

	return NULL;
}

/*
 * @brief: Function will be operated when the "dismissed" signal will be called
 * @param[data]: Data needed in this function
 * @param[obj]: Smart object
 * @param[event_info]: Information of event
 */
static void _popup_hide_finished_cb(void *data, Evas_Object *obj, void *event_info)
{
	if (obj == NULL)
		return;

	/*
	 * Delete popup object in this function.
	 */
	evas_object_del(obj);
}

/*
 * @brief: Function will be operated when the H/W Back Key event occurred
 * @param[data]: Data needed in this function
 * @param[obj]: Smart object
 * @param[event_info]: Information of event
 */
static void _popup_hide_cb(void *data, Evas_Object *obj, void *event_info)
{
	if (obj == NULL)
		return;

	/*
	 * Use this function to dismiss the popup with hide effect.
	 * When the popup is dismissed, the "dismissed" signal will be emitted.
	 */
	elm_popup_dismiss(obj);
}

/*
 * @brief: Make popup with theme
 * @param[parent]: Object to which you want to set popup
 * @param[timeout]: Timeout in seconds that control timer when the popup is hidden
 * @param[text]: Text will be written on the popup
 */
void view_create_text_popup(Evas_Object *parent, double timeout, const char *text)
{
	Evas_Object *popup = NULL;
	Evas_Object *popup_layout = NULL;

	if (parent == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "parent is NULL.");
		return;
	}

	popup = elm_popup_add(parent);
	elm_object_style_set(popup, "circle");
	evas_object_size_hint_weight_set(popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

	eext_object_event_callback_add(popup, EEXT_CALLBACK_BACK, _popup_hide_cb, NULL);
	/*
	 * Delete popup object in _popup_hide_finished_cb(), when the "dismissed" signal will be called.
	 */
	evas_object_smart_callback_add(popup, "dismissed", _popup_hide_finished_cb, NULL);

	popup_layout = elm_layout_add(popup);
	elm_layout_theme_set(popup_layout, "layout", "popup", "content/circle");

	elm_object_content_set(popup, popup_layout);

	elm_popup_timeout_set(popup, timeout);

	if (text) {
		view_set_text(popup_layout, "elm.text", text);
	}

	evas_object_show(popup);
}

